//Library to control an 8 Seven Segment LED Display with Keys and LEDs using TM1638
//Based upon this TM1638 Library on GitHub https://github.com/rjbatista/tm1638-library
#pragma once
#include "B4RDefines.h"
#include "TM1638.h"
//~Author: Robert W.B. Linn
//~Version: 1.1
namespace B4R {
	//~shortname: TM1638
	//~Event: KeyPressed(Keys as Byte) 
	class B4RTM1638 {
		private:
			uint8_t beTM1638[sizeof(TM1638)];
			TM1638* module;
			Byte CurrentValue;
			//~hide
			void (*Event)(Byte NewValue);
			static void looper(void* b); 
		public:

			/**
			*Define the LED colors.
			*Note: Depending module type, the color green is not supported.
			*/
			#define /*Byte TM1638_COLOR_NONE;*/ B4RTM1638_TM1638_COLOR_NONE 0
			#define /*Byte TM1638_COLOR_RED;*/ B4RTM1638_TM1638_COLOR_RED 1
			#define /*Byte TM1638_COLOR_GREEN;*/ B4RTM1638_TM1638_COLOR_GREEN 2

	    /** 
	    *Instantiate a TM1638 module by specifying the:
			*DataPin - Pin DIO to connect to Digital Pin
			*ClockPin - Pin CLK to connect to Digital Pin
			*StrobePin -Pin STB to connect to Digital Pin
			*ActivateDisplay - Set to true to show the display
			*Intensity - Set the intensity to 0 - 7
			*Example:<code>
			*Public tm As TM1638
			*'Init the device with data pin 8, clock pin 9 and strobe pin 7, show leading zeros, low intensity
			*tm.Initialize(8,9,7,True,2)
			*</code>
	    */
			void Initialize(Byte dataPin, Byte clockPin, Byte strobePin, bool activateDisplay, Byte intensity);

    	/**
    	*Set the display to an unsigned hexadecimal number (with or without leading zeros).
			*Example:<code>
			*'Display HEX and set the left 4 dots on (F0)
			*'Result: 1.2.3.4.ABCD
			*tm.SetDisplayToHexNumber(0x1234ABCD, 0xF0, False)
			*</code>
    	*/
    	void SetDisplayToHexNumber(ULong number, Byte dots, bool leadingZeros);
    
    	/**
    	*Set the display to an unsigned decimal number (with or without leading zeros).
			*Example:<code>
			*'Display HEX 1111 as DEC with no dots and no leading zeros
			*'Result: 4369
			*tm.SetDisplayToDecNumber(0x1111, 0x00, False)
			*</code>
    	*/
    	void SetDisplayToDecNumber(ULong number, Byte dots, bool leadingZeros);

	    /**
	    *Set the display to a signed decimal number (with or without leading zeros).
			*Example:<code>
			*'Display DEC 9081958 with leading zeros and no dots
			*'Result: 09081958
			*tm.SetDisplayToSignedDecNumber(9081958,0,True)
			*</code>
	    */
    	void SetDisplayToSignedDecNumber(Long number, Byte dots, bool leadingZeros);

	    /**
	    *Set the display to an unsigned binary number.
			*Example:<code>
			*'Display HEX 59 as digital with dots (FF)
			*'Result: 0.1.0.1.1.0.0.1
			*tm.SetDisplayToBinNumber(0x59, 0xFF)
			*</code>
	    */
    	void SetDisplayToBinNumber(Byte number, Byte dots);

	    /**
	    *Set the LED at pos to color (TM1638_COLOR_RED, TM1638_COLOR_GREEN or both)
			*Note: Depending module type, the color green is not supported.
			*Example:<code>
			*'Turn LED3 ON
			*tm.SetLED(tm.TM1638_COLOR_RED, 2)
			*'Turn LED3 OFF
			*tm.SetLED(tm.TM1638_COLOR_NONE, 2)
			*</code>
	    */
    	void SetLED(Byte color, Byte pos);

	    /**
	    *Set LEDs HIGH or LOW.
			*'Turn the 4 right LEDs ON (LED5-8)
			*tm.SetLEDs(0xF0)
			*'Turn the 4 left LEDs ON (LED1-4)
			*tm.SetLEDs(0x0F)
	    */
    	void SetLEDs(UInt led);

    	/**
    	*Returns the pressed buttons as a bit set (left to right).
			*This is handled by the KeyPress Listener event.
    	*/
    	//~hide
    	Byte GetButtons();

			/**
			*Adds a key pressed listener.
			*The KeyPressed event will be raised when one of the keys S1 - S8 is pressed.
			*Key Return Values S1 = 1, S2 = 2, S3 = 4, S4 = 8, S5 = 16, S6 = 32, S7 = 64, S8 = 128
			*Example:<code>
			*Private Sub Button_Pressed(Keys As Byte)
			*	Log("KeyPressed:", Keys)
			*End Sub
			*</code>
			*/
			void AddListener(SubVoidByte KeyPressedSub); 

			//Next functions taken from TM16XX.h

			/**
			*Clear the display.
			*/
			void ClearDisplay();

			/**
			*Set the display to an error message
			*/
			void SetDisplayToError();

	    /**
	    *Display a String with max 8 characters using built in font.
			*Example:<code>
			*Dim s As String = "Hello"
			*tm.SetDisplayToString(s)
			*</code>
	    */
			void SetDisplayToString(B4RString* string);

			/**
			*Set a single display at pos (starting at 0) to a digit (left to right).
			*Example:<code>
			*'Display digit 9 at the 4th LED which is pos 3 as position are 0 - 7
			*tm.SetDisplayDigit(9,3,False)
			*</code>
			*/
			void SetDisplayDigit(Byte digit, Byte pos, bool dot);

			/**
			*Clear a single display at pos (starting at 0, left to right)
			*Example:<code>
			*'Clear at the 4th LED which is pos 3 as position are 0 - 7
			*tm.ClearDisplayDigit(3,False)
			*</code>
			*/ 
    	void ClearDisplayDigit(Byte pos, bool dot);

			/**
			*Set the display (segments and LEDs) active or off and intensity (range from 0-7).
			*/
    	void SetupDisplay(bool active, Byte intensity);
 
	};
}